/*
 * Ambient - the ultimate desktop
 * ------------------------------
 *  2001-2004 by David Gerber <zapek@morphos.net>
 * All Rights Reserved
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
 *
 * $Id: lib.c,v 1.5 2006/08/08 13:31:42 fab Exp $
 */

#include "globals.h"

#include "lib.h"
#include "copyright.h"
#include "about.h"

/*
 * Ah well, I'm tired of cut & past so define the stuff
 * and copy the file somewhere else.
 */
#define LIBNAME      "about.amod"
#define LIBCOPYRIGHT " 2003-2004 David Gerber,  2005-2006 Ambient Open Source Team"
#define LIBPRI 0
#define LIBBASE AboutBase

char verstr[] = { "$VER: " LIBNAME " " LVERTAG " " LIBCOPYRIGHT };

extern ULONG LibFuncTable[];

struct Library*	LIB_Init(struct LibBase	*LIBBASE, BPTR SegList, struct ExecBase *SBase);

struct libinitstruct
{
	ULONG	LibSize;
	void	*FuncTable;
	void	*DataTable;
	void	(*InitFunc)(void);
};

struct libinitstruct libinit_struct =
{
	sizeof(struct LibBase),
	LibFuncTable,
	NULL,
	(void (*)(void))&LIB_Init
};


struct Resident libresident = {
	RTC_MATCHWORD,
	&libresident,
	&libresident + 1,
	RTF_PPC | RTF_EXTENDED | RTF_AUTOINIT | RTF_AFTERDOS,
	VERSION,
	NT_LIBRARY,
	0,
	LIBNAME,
	LIBNAME " " LVERTAG " "LIBCOPYRIGHT,
	&libinit_struct,
	/* new fields */
	REVISION,
	NULL,
};

/*
 * To tell the loader that this is a new emulppc elf and not
 * one for the ppc.library.
 */
ULONG __abox__ = 1;


/*
 * Library functions (system)
 */
struct Library*	LIB_Init(struct LibBase *LIBBASE, BPTR SegList, struct ExecBase *SBase)
{
	LIBBASE->SegList = SegList;
	LIBBASE->SBase = SBase;

	if (lib_init(SBase))
	{
		LIBBASE->Lib.lib_Node.ln_Pri = LIBPRI;

		return (&LIBBASE->Lib);
	}

	lib_cleanup();

	FreeMem((APTR)((ULONG)(LIBBASE) - (ULONG)(LIBBASE->Lib.lib_NegSize)),
		LIBBASE->Lib.lib_NegSize + LIBBASE->Lib.lib_PosSize);
	
	return (0);
}


/*
 * The following is needed because it's also called by LIB_Close() with
 * PPC args
 */
static ULONG libexpunge(struct LibBase *LIBBASE)
{
	BPTR MySegment;

	MySegment =	LIBBASE->SegList;

	if (LIBBASE->Lib.lib_OpenCnt)
	{
		LIBBASE->Lib.lib_Flags |= LIBF_DELEXP;
		return (NULL);
	}

	Forbid();
	Remove((struct Node *)&LIBBASE->Lib);
	Permit();

	lib_cleanup();

	FreeMem((APTR)((ULONG)(LIBBASE) - (ULONG)(LIBBASE->Lib.lib_NegSize)),
		LIBBASE->Lib.lib_NegSize + LIBBASE->Lib.lib_PosSize);

	return ((ULONG)MySegment);
}


ULONG LIB_Expunge(void)
{
	struct LibBase *LIBBASE = (struct LibBase *)REG_A6;

	return (libexpunge(LIBBASE));
}


static ULONG opened;

struct Library * LIB_Open(void)
{
	struct LibBase	*LIBBASE = (struct LibBase *)REG_A6;

	if (!opened)
	{
		opened = 1;
		
		if (!lib_open())
		{
			lib_cleanup();
			return (0);
		}
	}

	LIBBASE->Lib.lib_Flags &= ~LIBF_DELEXP;
	LIBBASE->Lib.lib_OpenCnt++;
	return (&LIBBASE->Lib);
}


ULONG LIB_Close(void)
{
	struct LibBase	*LIBBASE = (struct LibBase *)REG_A6;

	if ((--LIBBASE->Lib.lib_OpenCnt) == 0)
	{
		if (LIBBASE->Lib.lib_Flags & LIBF_DELEXP)
		{
			return (libexpunge(LIBBASE));
		}
	}
	return (0);
}


ULONG LIB_Reserved(void)
{
	return (0);
}

